@@ -14,6 +14,7 @@ __PACKAGE__->mk_accessors(qw/aws_secret_access_key aws_access_key_id token regio
 use Carp;
 use Data::Dumper;
 use Date::Format;
+use English qw {-no_match_vars};
 use Exporter;
 use File::chdir;
 use File::HomeDir;
@@ -30,7 +31,7 @@ use constant  AWS_CONTAINER_CREDENTIALS_URL          => 'http://169.254.170.2';
 
 use vars qw/$VERSION @EXPORT/;
 
-$VERSION = '1.0.14'; $VERSION=~s/\-.*$//;
+$VERSION = '1.0.15'; $VERSION=~s/\-.*$//;
 
 @EXPORT = qw/$VERSION/;
 
@@ -48,165 +49,6 @@ $VERSION = '1.0.14'; $VERSION=~s/\-.*$//;
   };
 }
 
-=pod
-
-=head1 NAME
-
-C<AWS::Credentials>
-
-=head1 SYNOPSIS
-
- my $aws_creds = Amazon::Credentials->new({order => [qw/env file container role/]});
-
-=head1 DESCRIPTION
-
-Class to find AWS credentials from either the environment,
-configuration files, instance meta-data or container role.
-
-You can specify the order using the C<order> option in the constructor
-to determine the order in which the class will look for credentials.
-The default order is I<environent>, I<file>, I<container>, I<instance
-meta-data>. See L</new>.
-
-=head1 METHODS
-
-=head2 new
-
- new( options );
-
- my $aws_creds = Amazon::Credential->new( { profile => 'sandbox', debug => 1 });
-
-C<options> is a hash of keys that represent options.  Any of the
-options can also be retrieved using their corresponding 'get_{option}
-method.
-
-Options are listed below.
-
-=over 5
-
-=item aws_access_key_id
-
-AWS access key.
-
-=item aws_secret_access_key
-
-AWS secret access key.
-
-I<Note: If you pass the access keys in the constructor then the
-constructor will not look in other places for credentials.>
-
-=item debug
-
-Set to true for verbose troubleshooting information.
-
-=item logger
-
-Pass in your own logger that has a C<debug()> method.  Otherwise the
-default logger will output debug messages to STDERR.
-
-=item user_agent
-
-Pass in your own user agent, otherwise LWP will be used. I<Probably>
-only useful to override this for testing purposes.>
-
-=item profile
-
-The profile name in the configuration file (F<~/.aws/config> or
-F<~/.aws/credentials>).
-
- my $aws_creds = Amazon::Credentials->new({ profile => 'sandbox' });
-
-The class will also look for the environment variable C<AWS_PROFILE>,
-so you can invoke your script like this:
-
- $ AWS_PROFILE=sandbox my-script.pl
-
-=item order
-
-An array reference containing tokens that specifies the order in which the class will
-search for credentials.
-
-default:  env, role, container, file
-
-Example:
-
-  my $creds = Amazon::Credentials->new( { order => [ qw/file env role/] });
-
-=over 10
-
-=item env - Environment
-
-If there exists an environment variable $AWS_PROFILE, then an attempt
-will be made to retrieve credentials from the credentials file using
-that profile, otherwise the class will for these environment variables
-to provide credentials.
-
- AWS_ACCESS_KEY_ID
- AWS_SECRET_ACCESS_KEY
- AWS_SESSION_TOKEN
-
-I<Note that when you set the environment variable AWS_PROFILE, the
-order essentially is overridden and we'll look in your credential
-files (F<~/.aws/config>, F<~/.aws/credentials>) for your credentials.>
-
-=item file - Configuration Files
-
-=over 15
-
-=item ~/.aws/config
-
-=item ~/.aws/credentials
-
-=back
-
-The class will attempt to find the credentials in either of these two
-files.  You can also specify a profile to use for looking up the
-credentials by passing it into the constructor or setting it the
-environment variable C<AWS_PROFILE>.  If no profile is provided, the
-default credentials or the first profile found is used.
-
- my $aws_creds = Amazon::Credentials->new({ order => [qw/environment role file/] });
-
-=item container - Task Role
-
-If the process is running in a container, the container may have a
-task role.  We'll look credentials using the container metadata
-service.
-
- http://169.254.170.2/$AWS_CONTAINER_CREDENTIALS_RELATIVE_URI
-
-=item role - Instance Role
-
-The class will use the
-I<http://169.254.169.254/latest/meta-data/iam/security-credential> URL
-to look for an instance role and credentials.
-
-Credentials returned by accessing the meta-data include a token that
-should be passed to Amazon APIs along with the access key and secret.
-That token has an expiration and should be refreshed before it
-expires.
-
- if ( $aws_creds->is_token_expired() ) {
-   $aws_creds->refresh_token()
- }
-
-=back
-
-=item region
-
-Default region. The class will attempt to find the region in either
-the configuration files or the instance unless you specify the region
-in the constructor.
-
-=item timeout
-
-When looking for credentials in metadata URLs, this parameter
-specifies the timeout value for C<LWP>.  The default is 3 seconds.
-
-=back
-
-=cut
-
 sub new {
   my $class = shift;
   my $self = $class->SUPER::new(ref($_[0]) ? $_[0] : { @_ });
@@ -234,22 +76,6 @@ sub new {
   $self;
 }
 
-=pod
-
-=head2 get_default_region
-
-Returns the region of the currently running instance.  The constructor
-will set the region to this value unless you set your own C<region>
-value.  Use C<get_region> to retrieve the value after instantiation or
-you can call this method again and it will make a second call to
-retrieve the instance metadata.
-
-You can also invoke this as a class method:
-
- $ AWS_REGION=$(perl -MAmazon::Credentials -e 'print Amazon::Credentials::get_default_region;')
-
-=cut
-
 sub get_default_region {
   my $self = shift;
   
@@ -293,21 +119,6 @@ sub set_credentials {
   }
 }
 
-=pod
-
-=head2 get_ec2_credentials (deprecated)
-
-=head2 find_credentidals
-
- find_credentials( option => value, ...);
-
-You normally don't want to use this method. It's automatically invoked
-by the constructor if you don't pass in any credentials. Accepts a
-hash or hash reference consisting of keys (C<order> or C<profile>) in
-the same manner as the constructor.
-
-=cut
-
 sub get_ec2_credentials {
   goto &find_credentials;
 }
@@ -329,12 +140,14 @@ sub find_credentials {
   foreach my $location (@order) {
     
     if ( $location eq 'env' ) {
+      
       if ( $ENV{AWS_ACCESS_KEY_ID} && $ENV{AWS_SECRET_ACCESS_KEY} ) {
         @{$creds}{qw/source aws_access_key_id aws_secret_access_key token/}
           = (
           'ENV',
           @ENV{qw/AWS_ACCESS_KEY_ID AWS_SECRET_ACCESS_KEY AWS_SESSION_TOKEN/}
-          );
+            );
+        $creds->{source} = 'env';
       } ## end if ( $ENV{AWS_ACCESS_KEY_ID...})
     } ## end if ( $location eq 'env')
     elsif ( $location eq 'container' ) {
@@ -365,6 +178,8 @@ sub find_credentials {
 
         # look for credentials...by interating through credentials file
         while (my $current_line = <$fh>) {
+          $self->get_logger->debug('looking for profile ' . ($profile_name // '') . "\n");
+            
           chomp $current_line;
 
           # once we find a profile section that matches, undef it
@@ -396,6 +211,18 @@ sub find_credentials {
           elsif ( defined $profile_name && $current_line =~/^\s*\[\s*profile\s+/ ) {
             last;
           }
+          elsif ($current_line =~/credential_process/ ) {
+            my $process;
+            if ( $current_line =~/\=(.*)$/ ) {
+              $process = $1;
+            }
+            
+            @{$creds}{qw{aws_access_key_id aws_secret_access_key token}} =
+              $self->get_credentials_from_process($process);
+
+            $creds->{source} = 'process';
+            last;
+          }
           elsif ($current_line =~/^\s*aws_secret_access_key\s*=\s*(.*)$/) {
             last if defined $creds->{aws_secret_access_key};    # next profile
             $creds->{aws_secret_access_key} = $1;
@@ -427,19 +254,6 @@ sub find_credentials {
   return $creds;
 } ## end sub find_credentials
 
-=pod
-
-=head2 is_token_expired
-
- is_token_expired( window-interval )
-
-Returns true if the token is about to expire (or is
-expired). C<window-interval> is the time in minutes before the actual
-expiration time that the method should consider the token expired.
-The default is 5 minutes.  Amazon states that new credentials will be
-available I<at least> 5 minutes before a token expires.
-
-=cut
 
 sub is_token_expired {
   my $self = shift;
@@ -489,44 +303,6 @@ sub _iso8601_to_time {
   return $gmtime;
 }
 
-=pod
-
-=head2 get_creds_from_role
-
- get_creds_from_role()
-
-Returns a hash, possibly containing access keys and a token.
-
-=over 5
-
-=item aws_access_key_id
-
-The AWS access key.
-
-=item aws_secret_access_key
-
-The AWS secret key.
-
-=item token
-
-Security token used with access keys.
-
-=item expiration
-
-Token expiration date.
-
-=item role
-
-IAM role if available.
-
-=item source
-
-Will be 'IAM' if role and credentials found.
-
-=back
-
-=cut
-
 sub get_creds_from_role {
   my $self = shift;
 
@@ -576,20 +352,27 @@ sub get_creds_from_role {
   $creds;
 }
 
-=pod
+sub get_credentials_from_process {
+  my ($self, $process) = @_;
 
-=head2 refresh_token
+  open(my $fh, '-|', $process)
+    or die "could not open pipe to $process\n";
 
- refresh_token() (deprecated)
- refresh_credentials()
-
-Retrieves a fresh set of IAM credentials.
-
- if ( $creds->is_token_expired ) {
-   $creds->refresh_token()
- }
-
-=cut
+  local $RS;
+    
+  my $credentials = <$fh>;
+  close $fh;
+ 
+  $credentials = eval {
+    JSON::PP->new->decode($credentials);
+  };
+  
+  if ( $EVAL_ERROR || !$credentials ) {
+    die "could not get credentials from process\n$EVAL_ERROR\n";
+  }
+  
+  return @{$credentials}{qw/AccessKeyId SecretAccessKey SessionToken/};
+}
 
 sub refresh_credentials {
   goto &refresh_token;
@@ -613,25 +396,6 @@ sub refresh_token {
 }
 
 
-=pod
-
-=head2 get_creds_from_container
-
- get_creds_from_container()
-
-Retrieves credentials from the container's metadata at
-http://169.254.170.2.  Returns a hash of credentials containing:
-
-  aws_access_key_id                                                                                                                             
-  aws_secret_access_key
-  aws_session_token
-
-Returns an empty hash if no credentials found.  The environment
-variable C<AWS_CONTAINER_CREDENTIALS_RELATIVE_URI> must exist or you
-must pass the value of the path as an argument.
-
-=cut
-
 sub get_creds_from_container {
   my ($self, $uri) = @_;
 
@@ -685,8 +449,288 @@ sub get_creds_from_container {
   return $creds;
 } ## end sub get_creds_from_container
 
+
+1;
+
+__END__
+
 =pod
 
+=head1 NAME
+
+ AWS::Credentials
+
+=head1 SYNOPSIS
+
+ my $aws_creds = Amazon::Credentials->new({order => [qw/env file container role/]});
+
+=head1 DESCRIPTION
+
+Class to find AWS credentials from either the environment,
+configuration files, instance meta-data or container role.
+
+You can specify the order using the C<order> option in the constructor
+to determine the order in which the class will look for credentials.
+The default order is I<environent>, I<file>, I<container>, I<instance
+meta-data>. See L</new>.
+
+=head1 METHODS
+
+=head2 new
+
+ new( options );
+
+ my $aws_creds = Amazon::Credential->new( { profile => 'sandbox', debug => 1 });
+
+C<options> is a hash of keys that represent options.  Any of the
+options can also be retrieved using their corresponding 'get_{option}
+method.
+
+Options are listed below.
+
+=over 5
+
+=item aws_access_key_id
+
+AWS access key.
+
+=item aws_secret_access_key
+
+AWS secret access key.
+
+I<Note: If you pass the access keys in the constructor then the
+constructor will not look in other places for credentials.>
+
+=item container - Task Role
+
+If the process is running in a container, the container may have a
+task role.  We'll look credentials using the container metadata
+service.
+
+ http://169.254.170.2/$AWS_CONTAINER_CREDENTIALS_RELATIVE_URI
+
+=item debug
+
+Set to true for verbose troubleshooting information.  Set C<logger> to
+a logger that implements a logging interface (ala C<Log::log4perl>.
+
+=item env - Environment
+
+If there exists an environment variable $AWS_PROFILE, then an attempt
+will be made to retrieve credentials from the credentials file using
+that profile, otherwise the class will for these environment variables
+to provide credentials.
+
+ AWS_ACCESS_KEY_ID
+ AWS_SECRET_ACCESS_KEY
+ AWS_SESSION_TOKEN
+
+I<Note that when you set the environment variable AWS_PROFILE, the
+order essentially is overridden and we'll look in your credential
+files (F<~/.aws/config>, F<~/.aws/credentials>) for your credentials.>
+
+=item file - Configuration Files
+
+=over 15
+
+=item ~/.aws/config
+
+=item ~/.aws/credentials
+
+=back
+
+The class will attempt to find the credentials in either of these two
+files.  You can also specify a profile to use for looking up the
+credentials by passing it into the constructor or setting it the
+environment variable C<AWS_PROFILE>.  If no profile is provided, the
+default credentials or the first profile found is used.
+
+ my $aws_creds = Amazon::Credentials->new({ order => [qw/environment role file/] });
+
+=item logger
+
+Pass in your own logger that has a C<debug()> method.  Otherwise the
+default logger will output debug messages to STDERR.
+
+=item order
+
+An array reference containing tokens that specifies the order in which the class will
+search for credentials.
+
+default:  env, role, container, file
+
+Example:
+
+  my $creds = Amazon::Credentials->new( { order => [ qw/file env role/] });
+
+=over 10
+
+=item profile
+
+The profile name in the configuration file (F<~/.aws/config> or
+F<~/.aws/credentials>).
+
+ my $aws_creds = Amazon::Credentials->new({ profile => 'sandbox' });
+
+The class will also look for the environment variable C<AWS_PROFILE>,
+so you can invoke your script like this:
+
+ $ AWS_PROFILE=sandbox my-script.pl
+
+=item region
+
+Default region. The class will attempt to find the region in either
+the configuration files or the instance unless you specify the region
+in the constructor.
+
+=item role - Instance Role
+
+The class will use the
+I<http://169.254.169.254/latest/meta-data/iam/security-credential> URL
+to look for an instance role and credentials.
+
+Credentials returned by accessing the meta-data include a token that
+should be passed to Amazon APIs along with the access key and secret.
+That token has an expiration and should be refreshed before it
+expires.
+
+ if ( $aws_creds->is_token_expired() ) {
+   $aws_creds->refresh_token()
+ }
+
+=back
+
+=item timeout
+
+When looking for credentials in metadata URLs, this parameter
+specifies the timeout value for C<LWP>.  The default is 3 seconds.
+
+=item user_agent
+
+Pass in your own user agent, otherwise LWP will be used. I<Probably>
+only useful to override this for testing purposes.>
+
+=back
+
+
+=head2 find_credentidals
+
+ find_credentials( option => value, ...);
+
+You normally don't want to use this method. It's automatically invoked
+by the constructor if you don't pass in any credentials. Accepts a
+hash or hash reference consisting of keys (C<order> or C<profile>) in
+the same manner as the constructor.
+
+=head2 get_creds_from_*
+
+These methods are called internally when the C<new> constructor is
+invoked. You should not normally call these methods
+
+=head3 get_creds_from_container
+
+ get_creds_from_container()
+
+Retrieves credentials from the container's metadata at
+http://169.254.170.2.  Returns a hash of credentials containing:
+
+  aws_access_key_id                                                                                                                             
+  aws_secret_access_key
+  aws_session_token
+
+Returns an empty hash if no credentials found.  The environment
+variable C<AWS_CONTAINER_CREDENTIALS_RELATIVE_URI> must exist or you
+must pass the value of the path as an argument.
+
+=head3 get_creds_from_process
+
+ get_creds_from_process(process)
+
+Retrieves credentials from a helper process defined in the config
+file. Returns the credentials tuple.
+
+=head3 get_creds_from_role
+
+ get_creds_from_role()
+
+Returns a hash, possibly containing access keys and a token.
+
+=over 5
+
+=item aws_access_key_id
+
+The AWS access key.
+
+=item aws_secret_access_key
+
+The AWS secret key.
+
+=item token
+
+Security token used with access keys.
+
+=item expiration
+
+Token expiration date.
+
+=item role
+
+IAM role if available.
+
+=item source
+
+Will be 'IAM' if role and credentials found.
+
+=back
+
+=head2 get_default_region
+
+Returns the region of the currently running instance.  The constructor
+will set the region to this value unless you set your own C<region>
+value.  Use C<get_region> to retrieve the value after instantiation or
+you can call this method again and it will make a second call to
+retrieve the instance metadata.
+
+You can also invoke this as a class method:
+
+ AWS_REGION=$(perl -MAmazon::Credentials -e 'print Amazon::Credentials::get_default_region;')
+
+=head2 get_ec2_credentials (deprecated)
+
+See L</find_credentials>
+
+=head2 is_token_expired
+
+ is_token_expired( window-interval )
+
+Returns true if the token is about to expire (or is
+expired). C<window-interval> is the time in minutes before the actual
+expiration time that the method should consider the token expired.
+The default is 5 minutes.  Amazon states that new credentials will be
+available I<at least> 5 minutes before a token expires.
+
+=head2 refresh_token
+
+ refresh_token() (deprecated)
+ refresh_credentials()
+
+Retrieves a fresh set of IAM credentials.
+
+ if ( $creds->is_token_expired ) {
+   $creds->refresh_token()
+ }
+
+=head1 SETTERS/GETTERS
+
+All of the parameters described in the new method can be accessed by a
+I<getter> or set using a I<setter>.
+
+=head1 CONTRIBUTING
+
+You can find this project on GitHub at
+L<https://github.com/rlauer6/perl-Amazon-Credentials>.  PRs always
+welcomed!
+
 =head1 AUTHOR
 
 Rob Lauer - <rlauer6@comcast.net>
@@ -698,5 +742,3 @@ is free software. It may be used, redistributed and/or modified under
 the same terms as Perl itself.
 
 =cut
-
-1;
